#include "ViewCell.h"
#include "Mesh.h"
#include "MeshKdTree.h"
#include "Triangle3.h"


ViewCell::ViewCell(Mesh *mesh): mMesh(mesh), mPvs(NULL) 
{
}

Mesh *ViewCell::GetMesh() 
{
	return mMesh;
}

BspPvs *ViewCell::GetPVS() 
{
	return mPvs;
}

AxisAlignedBox3 ViewCell::GetBox()
{
	return mMesh->mBox;
}
  
int ViewCell::CastRay(Ray &ray) 
{
	return 0;
}
  
bool ViewCell::IsConvex() 
{
	return mMesh->mIsConvex;
}

bool ViewCell::IsWatertight() 
{
	return mMesh->mIsWatertight;
}

float ViewCell::IntersectionComplexity() 
{
	return mMesh->mFaces.size();
}

int ViewCell::Type() const 
{ 
	return VIEWCELL; 
}

void  ViewCell::GetRandomSurfacePoint(Vector3 &point, Vector3 &normal) 
{
	point = Vector3(0,0,0);
}

void ViewCell::DeriveViewCells(const ObjectContainer &objects, 
							   ViewCellContainer &viewCells, 
							   const int max)
{
	int n = max > 0 ? std::min((int)objects.size(), max) : (int)objects.size();
	for (int i = 0; i < n; ++i)
	{
		Intersectable *object = objects[i];

		// extract the mesh instances
		if (object->Type() == Intersectable::MESH_INSTANCE)
		{
			MeshInstance *inst = dynamic_cast<MeshInstance *>(object);

			ViewCell *viewCell = new ViewCell(inst->GetMesh());
			viewCells.push_back(viewCell);
		}
	}
}